package com.aitrainer.api.controller

import com.aitrainer.api.model.ExercisePlan
import com.aitrainer.api.model.ExercisePlanDetail
import com.aitrainer.api.repository.ExercisePlanRepository
import org.slf4j.LoggerFactory
import org.springframework.http.HttpStatus
import org.springframework.http.ResponseEntity
import org.springframework.security.access.annotation.Secured
import org.springframework.web.bind.annotation.*
import javax.validation.Valid

@RestController
@RequestMapping("/api")
class ExercisePlanController( private val exercisePlanRepository: ExercisePlanRepository) {
    private val logger = LoggerFactory.getLogger(javaClass)

    @PostMapping("/exercise_plan")
    fun createNewExercisePlan(@Valid @RequestBody exercisePlan: ExercisePlan): ResponseEntity<ExercisePlan> {
        val newExercisePlan = exercisePlanRepository.save(exercisePlan)
        logger.info("Create new exercise plan: $newExercisePlan")
        return ResponseEntity.ok().body(newExercisePlan)
    }

    @GetMapping("/exercise_plan/last/{customerId}")
    fun getLastExercisePlan(@PathVariable(value = "customerId") customerId: Long): ResponseEntity<ExercisePlan> {

        val exercisePlan  = exercisePlanRepository.getLastExercisePlan(customerId)
        logger.info("Get last exercise plan by id $customerId: $exercisePlan")

        return if( exercisePlan != null)
            ResponseEntity.ok().body(exercisePlan) else
            ResponseEntity.notFound().build()
    }

    @Secured
    @PostMapping("/exercise_plan/{exercisePlanId}")
    fun updateExercisePlanById(@PathVariable(value = "exercisePlanId") exercisePlanId: Int,
                               @Valid @RequestBody exercisePlan: ExercisePlan,
                               @RequestHeader requestHeader: RequestHeader?
                               ): ResponseEntity<ExercisePlan> {
        val returnPlan = exercisePlanRepository.findById(exercisePlanId).orElse(null)
                ?: return ResponseEntity.notFound().build()
        val updatedPlan = returnPlan.copy(
                customerId = exercisePlan.customerId,
                name = exercisePlan.name,
                dateAdd = exercisePlan.dateAdd,
                description = exercisePlan.description
        )
        updatedPlan.exercisePlanId = exercisePlan.exercisePlanId
        logger.info("-- updateExercisePlanById id: $updatedPlan ")
        return ResponseEntity.ok().body(exercisePlanRepository.save(updatedPlan))
    }

    @Secured
    @PostMapping("/exercise_plan/delete/{exercisePlanId}")
    fun deleteExercisePlanById(@PathVariable(value = "exercisePlanId") exercisePlanId: Int)
            :ResponseEntity<HttpStatus> {
        val returnPlan = exercisePlanRepository.findById(exercisePlanId).orElse(null)
                ?: return ResponseEntity.notFound().build()
        try {
            exercisePlanRepository.delete(returnPlan)
        } catch (e: IllegalArgumentException ) {
            return ResponseEntity.ok().body(HttpStatus.BAD_REQUEST)
        }

        return ResponseEntity.ok().body(HttpStatus.OK)
    }
}