package com.aitrainer.api.openai

import com.aallam.openai.api.BetaOpenAI
import com.aallam.openai.api.chat.ChatCompletion
import com.aallam.openai.api.chat.ChatCompletionRequest
import com.aallam.openai.api.chat.ChatMessage
import com.aallam.openai.client.OpenAI
import com.aallam.openai.api.completion.CompletionRequest
import com.aallam.openai.api.completion.TextCompletion
import com.aallam.openai.api.http.Timeout
import com.aallam.openai.api.logging.LogLevel
import com.aallam.openai.api.model.Model
import com.aallam.openai.api.model.ModelId
import com.aallam.openai.client.OpenAIConfig
import com.google.gson.Gson
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.withContext
import org.springframework.beans.factory.annotation.Value
import org.springframework.stereotype.Service
import kotlin.time.Duration.Companion.seconds

@Service
class OpenAIService(@Value("\${openai.key}") private val openaiKey: String, private val modelName: String?, private val temperature: Double?) {

    private var openAI: OpenAI? = null
    var model: Model? = null

    private var modelId: ModelId? = null
    private suspend fun connect(modelName: String) {
        val config = OpenAIConfig(
            token = openaiKey,
            logLevel = LogLevel.All,
            timeout = Timeout(socket = 600.seconds)
        )
        openAI = OpenAI(config)
        modelId = ModelId(modelName)
        model = openAI!!.model(modelId!!)
    }


    /* models:
        gpt-3.5-turbo chat/completion
        gpt-4-0314 chat/completion
        text-davinci-003 completion
    */

    suspend fun completion(question: String): String {
        return withContext(Dispatchers.IO) {
            var realModelName = "text-davinci-003"
            if ( modelName != null) {
                realModelName = modelName
            }
            var realTemperature = 0.1
            if ( temperature != null ) {
                realTemperature = temperature
            }
            if (openAI == null) {
                connect(realModelName)
            }
            println("OpenAI Question: $question")
            val completionRequest = CompletionRequest(
                model = modelId!!,
                prompt = question,
                //echo = true,
                maxTokens = 2048 - question.length,
                temperature=realTemperature,
            )
            val completion: TextCompletion = openAI!!.completion(completionRequest)

            val result = completion.choices[0].text
            result
        }
    }

    @OptIn(BetaOpenAI::class)
    suspend fun chatCompletion(chatMessagesJson: String): String? {
        return withContext(Dispatchers.IO) {
            val gson = Gson()
            val messages = gson.fromJson(chatMessagesJson, Array<ChatMessage>::class.java).toList()
            val lastQuestion = messages.last().content

            var lengthQuestion = 0
            for ( message in messages ) {
                lengthQuestion += message.content.length
            }

            var realModelName = "gpt-3.5-turbo"
            if ( modelName != null) {
                realModelName = modelName
            }
            var realTemperature = 0.1
            if ( temperature != null ) {
                realTemperature = temperature
            }
            if (openAI == null) {
                connect(realModelName)
            }
            var tokens = 4096
            if (realModelName.contains("gpt-4")) {
                tokens = 8192
            }
            println("OpenAI Chat Last Question: $lastQuestion")
            val completionRequest = ChatCompletionRequest(
                model = ModelId(realModelName),
                messages = messages,
                maxTokens = tokens  - lengthQuestion,
                temperature = realTemperature,
            )
            val completion: ChatCompletion = openAI!!.chatCompletion(completionRequest)

            val result = completion.choices[0].message?.content
            print(result)
            result
        }
    }

    suspend fun getModels(): MutableList<String> {
        return withContext(Dispatchers.IO)  {
            val list: MutableList<String> = mutableListOf()
            if (openAI == null) {
                openAI = OpenAI(token = openaiKey)
            }


            openAI!!.models().forEach {
                println(it)
                list.add(it.id.id)
            }

            return@withContext list
        }
    }
}

