package com.aitrainer.api.test

import com.aitrainer.api.controller.CustomerPropertyController
import com.aitrainer.api.model.CustomerProperty
import com.aitrainer.api.repository.CustomerPropertyRepository
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.TestInstance
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.test.context.SpringBootTest
import org.springframework.http.HttpStatus
import kotlin.test.assertEquals
import kotlin.test.assertTrue

@SpringBootTest
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
class CustomerPropertyTest {

    @Autowired
    private lateinit var customerPropertyRepository: CustomerPropertyRepository


    @Test
    fun testGetAllCustomerProperties() {
        val controller = CustomerPropertyController(customerPropertyRepository )
        val response = controller.getAllByCustomerId(90)

        val properties = response.body

        assertTrue(properties is List)
        assertTrue(properties.isNotEmpty())
        assertEquals(properties.size, 9)
        assertEquals(properties[0].propertyValue, 81.0)
        assertEquals(properties[1].propertyValue, 82.0)
        assertEquals(properties[3].propertyId, 4)
    }

    @Test
    fun testGetLastCustomerProperties() {
        val controller = CustomerPropertyController(customerPropertyRepository)
        val response = controller.getLastPropertiesByCustomerId(90)

        val properties = response.body
        assertTrue(properties is List)
        assertTrue(properties.isNotEmpty())
        assertEquals(properties.size, 6)
        assertEquals(properties[2].propertyValue, 42.0)
        assertEquals(properties[0].propertyValue, 172.0)
    }

    @Test
    fun testInsert() {
        val customerProperty = CustomerProperty(
            customerId = 90,
            propertyId = 1,
            dateAdd = "2020-11-10",
            propertyValue = 81.5
        )

        val controller = CustomerPropertyController(customerPropertyRepository)
        val response = controller.createNewCustomerProperty(customerProperty)

        val newCustomerProperty = response.body

        assertTrue(newCustomerProperty is CustomerProperty)
        assertEquals(newCustomerProperty.propertyValue, 81.5)

        customerPropertyRepository.delete(customerProperty)
    }

    @Test
    fun testInsertGoal() {
        val customerGoal = CustomerProperty(
            customerId = 90,
            propertyId = 1,
            dateAdd = "2023-01-30",
            propertyValue = 76.0,
            goal = true,
            goalDate = "2023-05-31"
        )

        val controller = CustomerPropertyController(customerPropertyRepository)
        val response = controller.createCustomerGoal(customerGoal)



        val newCustomerProperty = response.body


        assertTrue(newCustomerProperty is CustomerProperty)
        val newGoal: CustomerProperty = customerPropertyRepository.findById(newCustomerProperty.customerPropertyId).orElse(null)
        assertTrue(true)
        assertEquals(newGoal.propertyValue, 76.0)
        assertEquals(newGoal.goal, true)
        assertEquals(newGoal.goalDate, "2023-05-31")

        customerPropertyRepository.delete(customerGoal)
    }

    @Test
    fun testUpdate() {
        var customerProperty = CustomerProperty(
            customerPropertyId = 91,
            dateAdd = "2021-02-15",
            propertyValue = 42.0
        )
        val controller = CustomerPropertyController(customerPropertyRepository)

        var response = controller.updateCustomerProperty(customerProperty.customerPropertyId, customerProperty)

        assertTrue(response.body is CustomerProperty)
        val updated = response.body
        assertEquals(updated!!.propertyValue, 42.0)
        updated.propertyValue = 81.0
        controller.updateCustomerProperty(customerProperty.customerPropertyId, customerProperty)

        //notFound
        customerProperty = CustomerProperty(
            customerPropertyId = 1000,
            dateAdd = "2021-02-15",
            propertyValue = 83.0
        )
        response = controller.updateCustomerProperty(customerProperty.customerPropertyId, customerProperty)
        assertEquals(response.statusCode, HttpStatus.NOT_FOUND)



    }

}